package RPM::Source::Tools::Dependency::Filter::DropSpecUndefinedVersionMacro;

use strict;
use warnings;
use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory::DependencyFilter;

our @ISA=q/RPM::Source::Tools::Dependency::Filter/;
our $verbose=1;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Drop Undefined Version Macro',
    -description => 'drop version if it is written as macro that is not defined in spec',
    -class => 'RPM::Source::Tools::Dependency::Filter::DropSpecUndefinedVersionMacro',
    -priority => FILTER_PRIORITY_POSTFILTER,
);


sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $version=$dep->{-version};
    return $dep unless defined $version;
    if ($version=~/^\%/) {
	my $expanded=$self->{-spec}->macros->macro_subst($version);
	if ($expanded eq '') {
	    print STDERR "info: dropped undefined $dep->{-version} in $dep->{-name} $dep->{-relation} $dep->{-version}\n" if $verbose;
	    delete $dep->{-name};
	    return;
	}
    }
    return $dep;
}

1;
