package RPM::Source::Convert::Plugins::ALT2ALT;
use strict;
use warnings;
use Carp;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory::DependencyFilter;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $cleanup_private_rpm_macros=1;
our $cleanup_req_devel=1;
our $cleanup_req_pymod=0;

our @LONGOPT=(
    'cleanup-private-rpm-macros!' => \$cleanup_private_rpm_macros,
    'cleanup-requires-devel!' => \$cleanup_req_devel,
    'cleanup-requires-python-modules!' => \$cleanup_req_pymod,
    );

sub print_longopt {
    print "ALT specific cleanup options:\n";
    print "	--cleanup-requires-python-modules. Cleanup Req: from python-module-*.\n";
    print "	--no-cleanup-requires-devel. Do not cleanup Req: from -devel.\n";
}

my @subst_table;
my $subst_data='
__awk		gawk
__bzip2	bzip2
__cat		cat
__chgrp	chgrp
__chmod	chmod
__chown	chown
__cp		cp
__cpio		cpio
__file		file
__gpg		gpg
__grep		grep
__gzip		gzip
__id		id
__install	install
__install_info	install-info
__ln_s		ln -s
__mkdir_p	mkdir -p
__subst	sed -i
__cpp		gcc -E
__make		make
__mkdir	mkdir
__mv		mv
__patch	patch
__perl		/usr/bin/perl
__pgp		pgp
__rm		rm
__rsh		rsh
__sed		sed
__ssh		ssh
__tar		tar
__unzip	unzip
__ar		ar
__as		as
__cc		gcc
__cxx		g++
__ld		ld
__nm		nm
__objcopy	objcopy
__objdump	objdump
__ranlib	ranlib
__strip	strip
__libtoolize	libtoolize
__aclocal	aclocal
__autoheader	autoheader
__automake	automake
__autoconf	autoconf
';
#__python	/usr/bin/python

{
    local $_;
    open my $strfh, '<', \$subst_data or die "$!";
    while (<$strfh>) {
	chomp;
	next if $_ eq '';
	my @line=split /\t+/;
	push @subst_table, @line[0,1] if $line[1];
	die "bad line $_\n" if @line>2 or @line<2; # and $line[0]!~/^\s*$/;
    }
    close $strfh;
}

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'ALT2ALT::BuildRequiresCleanup',
    -description => 'drop BuildRequires on make, pkgconfig, so on',
    -class => 'ALT2ALT::DependencyFilter::BuildRequiresCleanup',
    -tags => SPEC_TAG_BUILD_REQUIRES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'ALT2ALT::NoDevelRequire',
    -description => 'drop Requires: on *-devel packages',
    -class => 'RPM::Source::Tools::Dependency::Filter::ALT2ALT::NoDevelReqs',
    -tags => SPEC_TAG_REQUIRES,
    -priority => FILTER_PRIORITY_DEFAULT,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'ALT2ALT::NoPythonModuleReqs',
    -description => 'drop Requires: on python module names',
    -class => 'RPM::Source::Tools::Dependency::Filter::ALT2ALT::NoPythonModuleReqs',
    -tags => SPEC_TAG_REQUIRES,
    -priority => FILTER_PRIORITY_DEFAULT,
);


sub transformations {
    return unless $RPM::Source::Convert::Plugins::ALT2ALT::cleanup_private_rpm_macros;
    return
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'alt2alt_cleanup_private_rpm_macros',
	    -opt_handler_sub=>\&opt_handler_alt2alt_cleanup_private_rpm_macros);
}

my %shellsec=map {$_=>1} qw/prep build install/;
sub opt_handler_alt2alt_cleanup_private_rpm_macros {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    foreach my $section ($spec->get_sections()) {
	my $secname=$section->get_type();
	next unless $shellsec{$secname};
	$section->subst_body(qr'=\%__perl','=/usr/bin/perl');
	for (my $i=0;$i<@subst_table and $subst_table[$i+1];$i=$i+2) {
	    my $from = $subst_table[$i];
	    my $to=$subst_table[$i+1];
	    $section->subst_body(qr"(?<!%)\%\{?$from\}?(?=(?:\s|$))",$to);
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}


package RPM::Source::Tools::Dependency::Filter::ALT2ALT::NoDevelReqs;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    return $dep unless $RPM::Source::Convert::Plugins::ALT2ALT::cleanup_req_devel;
    my $pkgname=$dep->{-name};
    if ($pkgname=~/-devel$/) {
	delete $dep->{-name};
    }
    return $dep;
}


package RPM::Source::Tools::Dependency::Filter::ALT2ALT::NoPythonModuleReqs;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    return $dep unless $RPM::Source::Convert::Plugins::ALT2ALT::cleanup_req_pymod;
    my $pkgname=$dep->{-name};
    if ($pkgname=~/^python3?-module-/) {
	delete $dep->{-name};
    }
    return $dep;
}

package ALT2ALT::DependencyFilter::BuildRequiresCleanup;

use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;
our $verbose=0;

## TODO python ??
##
sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    print STDERR "DEBUG: BuildRequiresCleanup: testing [$pkgname]\n" if $verbose>1;
    return if grep {$pkgname eq $_} qw!
make
%{__make}
pkgconfig
!;
    return $dep;
}

1;
