package RPM::Source::Convert::Plugins::Alternatives2ALT;

use strict;
use warnings;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Tools::ShellGlob;
use RPM::Source::SpecConst;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;
our $verbose=1;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'alternatives2altlinux',
	    -opt_handler_sub=>\&opt_handler_convert_alternatives);
}

sub opt_handler_convert_alternatives {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'pre') {
	    #
	} elsif ($t eq 'post') {
	    &alt_generic_convert_alternatives($spec,$sec,$opt,$resourceobj);
	} elsif (($t eq 'postun') or ($t eq 'preun')) {
	    &alt_generic_convert_alternatives($spec,$sec,$opt,$resourceobj);
	} elsif ($t eq 'pretrans') {
	    #
	} elsif ($t eq 'posttrans') {
	    &alt_generic_convert_alternatives($spec,$sec,$opt,$resourceobj);
	# other scripts == triggers
	} elsif ($RPM::Source::SpecConst::SCRIPT_SECTION{$t}) {
	    # not good, no state :(
	    #&alt_generic_convert_alternatives($spec,$sec,$opt,$resourceobj);
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

# --------------------------------------------------------------
# alternatives conversion code;
# --------------------------------------------------------------
#

sub alt_generic_convert_alternatives {
    my ($spec,$section,$opt,$resourceobj)=@_;
    my @out;
    my $ptr = $section->get_bodyref();
    my $converted_alternatives;
    while (@$ptr>0) {
	my $line = shift @$ptr;
	if ($line=~/\s*#/) {
	    push @out, $line;
	} elsif ($line=~s,^(\s*)(?:(?:\%\{_sbindir\}|/usr/sbin)/)?(?:update-)?(?:\%\{__)?alternatives\}? --(\w+),,) {
	    my $preline=$1;
	    my $updcmd=$2;
	    push @out, &_convert_single_alternative($spec,$section,$ptr,$preline,$updcmd,$line);
	    $converted_alternatives=1;
	} elsif ($line=~s,^(\s*)(?:(?:\%\{_sbindir\}|/usr/sbin)/)?(?:update-)?(?:\%\{__)?alternatives\}?\s+\\\n$,,) {
	    my $preline=$1;
	    $line = shift @$ptr;
	    if ($line=~s/^\s*--(\w+)//) {
		my $updcmd=$1;
		push @out, &_convert_single_alternative($spec,$section,$ptr,$preline,$updcmd,$line);
		$converted_alternatives=1;
	    } else {
		unshift @$ptr, $line;
		push @out, $preline."# unknown update-alternatives\n";
	    }
	} else {
	    push @out, $line;
	}
    }
    $resourceobj->buildreq->add_pre_buildreq('rpm-macros-alternatives') if $converted_alternatives;
    $section->set_body(\@out);
}

sub _convert_single_alternative {
    my ($spec,$section,$ptr,$preline,$updcmd,$line)=@_;
    $line=&read_multiline($line, $ptr);
    # the end is abandoned :(
    if ($updcmd eq "remove") {
	my $name=&read_shell_token(\$line);
	# pre 0.4 alt alternatives
	#return $preline.'%unregister_alternatives '.&get_alternatives_file_name($spec,$section,$name)."\n";
	return $preline.":\n";
    } elsif ($updcmd eq "install") {
	my $link=&read_shell_token(\$line);
	my $name=&read_shell_token(\$line);
	my $path=&read_shell_token(\$line);
	my $priority=&read_shell_token(\$line);
	unless ($priority) {
	    $priority=10;
	    warn "alternative priority is 0 - set to $priority\n";
	}
	my @alternatives=($link,$path,$priority);
	&read_space(\$line);
	while ($line=~/--slave/) {
	    &read_shell_token(\$line);
	    my $slink=&read_shell_token(\$line);
	    my $sname=&read_shell_token(\$line);
	    my $spath=&read_shell_token(\$line);
	    push @alternatives, ($slink,$spath,$path);
	}
	&create_alternatives($spec,$section, &get_alternatives_file_name($spec,$section,$name),\@alternatives);
	return $preline.":\n";
    } else {
	warn "unknown update-alternatives --$updcmd";
	return $preline."# unknown update-alternatives --$updcmd\n";
    }
}

sub create_alternatives {
    my ($spec, $section, $alternatives_file_name,$alternativesptr) = @_;
    my @ghosts;
    my @text=('install -d $RPM_BUILD_ROOT/%_altdir; cat >$RPM_BUILD_ROOT/%_altdir/'.$alternatives_file_name.'<<EOF
');
    while (@$alternativesptr>0) {
	my $provided=shift(@$alternativesptr);
	my $alternative=shift(@$alternativesptr);
	my $weight=shift(@$alternativesptr);
	push @text, $provided."\t".$alternative."\t".$weight."\n";
	push @ghosts, $provided;
    }
    push @text, "EOF\n";
    my $installsec=$spec->add_section('install');
    $installsec->push_body(@text);
    my $filesec=$spec->get_section('files',$section->get_raw_package());
    if ($filesec) {
	$filesec->unshift_body('%_altdir/'.$alternatives_file_name."\n");
	# kill ghosts
	my $macros=$spec->macros;
	my @ghosts_expanded=map {$macros->macro_subst_if_defined($_)} @ghosts;
	$filesec->map_body(sub {
	    if (/\%ghost\s/) {
		my $line=$_;
		chomp $line;
		while ($line=~s/^\s*\%(?:ghost|verify\s*\([^\)]+\)|lang\s*\([^\)]+\))\s+//) {};
		$line=$macros->macro_subst_if_defined($line);
		print STDERR "debug:alternatives:cleanup: trying $line from $_" if $verbose>1;
		# shell match
		$line=~s/^\s*//;
		$line=~s/\s*$//;
		if ($line=~m!/\s*$! or $line=~m!\%dir!) {
		    print STDERR "debug:alternatives: ghost dir: $line\n";
		    return;
		}
		my $re=&RPM::Source::Tools::ShellGlob::spec_glob2pat($line);
		if ($verbose>2) {
		    print STDERR "debug:alternatives:cleanup: re=$re\n";
		    print STDERR "debug:alternatives:cleanup: ghosts=",join(',',@ghosts_expanded),"\n";
		}		# drop ghost line - it contains alternative
		if (grep {/^$re$/} @ghosts_expanded) {
		    print STDERR "info: dropped $_: alternative\n" if $verbose;
		    $_='';
		}
	    }
			   });
	# TODO: clean up %install from touch <ghost> and other manipulations, like chmod
	# why he did chmod'ing the ghost ? for %verify ?
	# touch $RPM_BUILD_ROOT%{_bindir}/whois
	# chmod 755 $RPM_BUILD_ROOT%{_bindir}/whois

    } else {
	warn "WARNING: section %files for [".$section->get_raw_package()."] not found: can't write alternatives found in ".$section->get_type()."!";    }
}

sub get_alternatives_file_name {
    my ($spec,$section,$name)=@_;
#    return $name.'_'.$section->get_package_name();
    my $pkg=$section->get_package_name();
    return $name.($pkg ? "_$pkg" : '');
}

sub read_multiline {
    my ($line, $ptr) =@_;
    while (@$ptr>0 && $line=~s/\\[\r\n]+?$//) {
	chomp $line;
#	print "MULTILINE:$line\n";
	my $appender=shift(@$ptr);
	# sometimes alternatives are conditional
	if ($appender=~/^(\%(?:if|else|endif).*)$/) {
	    # see lcgdm
	    warn "WARNING: found $1 in a body of an alternative: please,fix manually.\n";
	    $appender=shift(@$ptr);
	}
	$line.=$appender;
    }
    return $line;
}

sub read_shell_token {
    my ($lineptr) =@_;
    if ($$lineptr=~s/^\s*(\S+)//) {
	return $1;
    }
    return "";
}

sub read_space {
    my ($lineptr) =@_;
    $$lineptr=~s/^\s*//;
}

1;
