package RPM::Source::Convert::Plugins::DistroMap::Packages;
use strict;
use warnings;
use Carp;

use RPM::Source::Tools::PackageFilter;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;


our %PROV_SKIP = map {$_ => 1} qw!!;
our (%PROV_SUBST, %PROV_DROP_VER);

our %DEP_DROP_VER = map {$_ => 1} qw!!;

our %DEP_SKIP = map {$_ => 1} qw!!;

our %DEP_SUBST = (
'%{_sbindir}/update-alternatives' => 'alternatives',
'/usr/sbin/update-alternatives' => 'alternatives',
'%{_sbindir}/alternatives' => 'alternatives',
'/usr/sbin/alternatives' => 'alternatives',
);

our(%CONFL_SKIP, %CONFL_SUBST, %CONFL_DROP_VER, @CONFL_FILTERS);

our @COMMON_FILTERS;
our @PROV_FILTERS;
our @DEP_FILTERS;
our @REQ_FILTERS;
our @POSTREQ_FILTERS;

our $verbose=0;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'old_style_deps_of_packages_convert',
	    -opt_handler_sub=>\&opt_handler_old_style_deps_of_packages_convert);
}

sub opt_handler_old_style_deps_of_packages_convert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    foreach my $filter (@DEP_FILTERS, @COMMON_FILTERS, @PROV_FILTERS, @REQ_FILTERS, @POSTREQ_FILTERS) {
	$filter->init($resourceobj,$spec);
    }
    foreach my $sec ($spec->get_sections()) {
	&old_style_deps_of_cleanup_headers($sec) if $sec->get_type() eq 'package';
    }
    #print STDERR "old_style_deps_of_pkg_prehook_fix \$_=$_\n";
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub old_style_deps_of_cleanup_headers {
    my ($section)=@_;
    my @out;
    foreach (@{$section->get_bodyref()}) {
#	if (/^\s*((?:BuildReq|BuildRequires|PreReq|BuildPreReq|BuildPrereqs|Requires|Conflicts)\s*(?:\(\w+\))?)\s*:(\s*)(.*\S)\s*$/i) {
#%{?!_without_fuse:BuildRequires: fuse-devel}
	if (/^\s*(\%\{!?\?[^:]+:)?\s*((?:BuildReq|BuildRequires|PreReq|BuildPreReq|BuildPrereqs|Requires|Conflicts)\s*(?:\(\w+\))?)\s*:(\s*)(.*\S)\s*$/i) {
	    my $macro=$1;
	    my $Tag=$2;
	    my $space=$3;
	    my $reqlist=$4;
	    my $endmacro='';
	    if ($macro) {
		$reqlist=~s/\s*(\})\s*$//;
		$endmacro=$1;
	    } else {
		$macro='';
	    }
	    my $filtered;
	    if ($Tag!~/build/i and $Tag=~/req/i) {
		#print STDERR "REQ TAG: $Tag\n";
		$filtered = &filter_pkg_req_list($reqlist);
	    } elsif ($Tag=~/conflict/i) {
		#print STDERR "CONFL TAG: $Tag\n";
		$filtered = &filter_pkg_conflicts_list($reqlist);
	    } else {
		#print STDERR "DEP TAG: $Tag\n";
		$filtered = &filter_pkg_dep_list($reqlist);
	    }
	    print STDERR "Filtered $Tag: $filtered\n" if $verbose>2;
	    push @out, "$macro$Tag:$space$filtered$endmacro\n" if ($filtered);
	} elsif (/^\s*(Provides)\s*:(\s*)(.*\S)\s*$/i) {
	    my $Tag=$1;
	    my $space=$2;
	    my $filtered = &filter_pkg_prov_list($3);
	    push @out, "$Tag:$space$filtered\n" if ($filtered);
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

sub filter_pkg_prov_list {
    my ($line)=@_;
    return &RPM::Source::Tools::PackageFilter::filter_pkg_list($line, \%PROV_SKIP, \%PROV_SUBST, \%PROV_DROP_VER, [@PROV_FILTERS, @COMMON_FILTERS]);
};

sub filter_pkg_conflicts_list {
    my ($line)=@_;
    return &RPM::Source::Tools::PackageFilter::filter_pkg_list($line, \%CONFL_SKIP, \%CONFL_SUBST, \%CONFL_DROP_VER, [@CONFL_FILTERS, @COMMON_FILTERS]);
};

sub filter_pkg_dep_list {
    my ($line)=@_;
    return &RPM::Source::Tools::PackageFilter::filter_pkg_list($line, \%DEP_SKIP, \%DEP_SUBST, \%DEP_DROP_VER, [@DEP_FILTERS, @COMMON_FILTERS]);
};

sub filter_pkg_req_list {
    my ($line)=@_;
    return &RPM::Source::Tools::PackageFilter::filter_pkg_list($line, \%DEP_SKIP, \%DEP_SUBST, \%DEP_DROP_VER, [@REQ_FILTERS, @DEP_FILTERS, @COMMON_FILTERS,@POSTREQ_FILTERS]);
};


### EXAMPLE old format cleaner ###################
package Generic2ALT::PackageFilter::NoMacrosReqs;
use strict;
use warnings;
BEGIN {our @ISA=q/RPM::Source::Tools::PackageFilter/;}

sub map_pkg {
    my ($self, $pkgname)=@_;
    return [] if $pkgname=~/^rpm-macros-/;
    return 0;
}

1;
