package RPM::Source::Convert::Plugins::Fedora2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Tools::Dependency::Filter;
use Source::Shared::Resource::DistroMap;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Factory::DependencyFilter;
use RPM::Source::Convert::Plugins::Generic2ALT;
# after Generic2ALT; not to disturb the order
use RPM::Source::Convert::Plugins::FontUsage2ALT;
use RPM::Source::Convert::Plugins::FedoraFontPkg2ALT;
use RPM::Source::Convert::Plugins::PerlAutodepsUseStyle2PathStyle;
use RPM::Source::Convert::Plugins::Python;
use RPM::Source::Convert::Plugins::OtherLang;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

# fake section to be processed in &convert_font_pkg
# a method to protect multisection macro from killing as the part of %clean section
use RPM::Source::SpecSection;
$RPM::Source::SpecSection::SECTION_MACROS{'_font_pkg'}=1;

$DistroMap::default_originrepo='fedora';

our $verbose=0;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Fedora Specific',
    -description => 'fedora specific dependency filter',
    -class => 'RPM::Source::Tools::Dependency::Filter::FedoraMapSpecificDeps',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'MacrosCleanup',
    -description => 'Macros, like %_isa',
    -class => 'RPM::Source::Tools::Dependency::Filter::MacrosCleanup',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS | SPEC_TAG_PROVIDES | SPEC_TAG_OBSOLETES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Fedora MimeHandler Deps',
    -description => 'drop Fedora Kmod and MimeHandler dependencies',
    -class => 'RPM::Source::Tools::Dependency::Filter::FedoraDropNotImplementedDeps',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);


sub register_messages {
    my ($self,$extopt)=@_;
#    RPM::Source::Transformation::Component::add_opt($extopt, {
#	});
}

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'fedora2alt_preconvert_generic',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_preconvert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'fedora2alt_convert',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_convert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_POSTCONVERT,
	    -name => 'fedora2alt_kill_python_duplicates',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_kill_python_duplicates),
	;
}

my $FEDORA_VERSION=30;

# before DistroMap::depencencies
sub opt_handler_fedora2alt_preconvert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;

    my $macros=$spec->macros();
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	next if $t eq 'changelog';
	if ($t eq 'package') {
	    # fedora perl packaging
	    #Requires:       perl(:MODULE_COMPAT_%(eval "`%{__perl} -V:version`"; echo $version))
	    #Requires:       perl(:MODULE_COMPAT_%(eval $(%{__perl} -V:version); echo $version))
	    #Requires:       perl(:MODULE_COMPAT_%{perl_version}.pm)
	    $sec->map_body(sub { 
		s/^(?:Build)?Requires:\s*$// if s/perl\(:MODULE_COMPAT_\%\(eval.+\)\)// or
		    s/perl\(:MODULE_COMPAT_\%\{?perl_version\}?(?:\.pm)?\)//;
		s/^(?:Build)?Requires:\s*$// if s/postgresql-server\(:MODULE_COMPAT_\%\{postgresql_major\}\)//;
		# ALT have no such macros :(
		s/(?<!%)\%\{sparc\}//g if /(?:Exclusive|Exclude|Build)Arch/ or /\%ifarch/;
		# nodejs support
		$_='' if /ExclusiveArch:\s+\%\{(?:go_arches|nodejs_arches|qt5_qtwebengine_arches|fpc_arches)\}/;
		s,\%\{\??nodejs_find_provides_and_requires\},,;
		s,%tests_req,BuildRequires:,;
			   } );
	} elsif ($t eq 'check') {
	    $sec->map_body(sub { 
		s!\%\{?_rpmconfigdir\}?/check-rpaths.*!!;#/;

		# logoved rule is fedora-debuginfo-list: can be moved to logoved
		# fc-specific perl; in altlinux there are no *.list
		##These are empty and interfere with the distribution test
		##rm *.list
		s!^(\s*rm\s+\*.list\b)!$1 ||:!;

			   } );


	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}


sub opt_handler_fedora2alt_convert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    my $mainsec=$spec->main_section;
    $mainsec->subst_body_if(qr'\%\{x86\}','%{ix86}',qr'^ExclusiveArch:');

    my ($flag_need_fedora_compat, $flag_need_cmake);
    my ($flag_macro_fedoraver, $flag_macro_texmf, 
	$flag_macro_kde, $flag_macro_kf5,
	$flag_macro_erlang, $flag_macro_nodejs, $flag_qt4_devel, $flag_qt5_devel, 
	$flag_macro_emacs, $flag_macro_go, $flag_macro_php,
	);
    my ($filter_from_provreq_is_used,$configure_macro_is_used);
    my @__push_mainsec;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	next if $t eq 'changelog';
	if ($t eq 'package') {
	    $sec->map_body(
		sub { 
		    if (/^Requires.*:\s+maven-local\s*$/ and $spec->macros->macro_subst('%name') ne 'javapackages-tools') {
			$_='';
			print STDERR "info: dropped Req: maven-local\n" if $verbose;
		    }
		    # FC specific hacks with __os_install_post / __arch_install_post, will break alt rpm
		    s,^\s*\%(global\s+__(?:arch|os)_install_post),#$1,;
		} );

	    # group uncovering; tmp code till conditional macros handling will be improved
	    #%if 0%{?rhel}
	    #Group: Development/Languages
	    #%endif
	    my (@new_body,@tmp_storage);
	    foreach my $line (@{$sec->get_bodyref()}) {
		if ($line=~/^\s*\%if\s+0\%\{\?rhel\}/) {
		    push @tmp_storage, $line;
		} elsif (@tmp_storage) {
		    if ($line=~/^\s*Group:/) {
			push @new_body, $line, @tmp_storage;
		    } else {
			push @new_body, @tmp_storage, $line;
		    }
		    @tmp_storage=();
		} else {
		    push @new_body, $line;
		}
	    }
	    $sec->set_body(\@new_body);
	} elsif ($t eq 'build') {
	    my $sec_has_qmake=0;
	    $sec->map_body(sub {
		s!(?:\.|source)\s+(?:/etc|\%\{_sysconfdir\})/profile.d/qt.sh!. /etc/profile.d/qt3dir.sh; export PATH=%_libdir/qt3/bin:\$PATH!;
		$flag_qt4_devel=1 if s,\%\{?_qt4_qmake\}?,qmake-qt4,;
		$flag_qt4_devel=1 if /qmake-qt4/;
		$flag_qt5_devel=1 if s,\%\{?_qt5_qmake\b\}?,qmake-qt5,;
		$flag_qt5_devel=1 if s,\%\{?qmake_qt5\b\}?,qmake-qt5,;
		$flag_qt5_devel=1 if /qmake-qt5/;
		$sec_has_qmake=1 if /^\s*qmake(?:\s|$)/;
		s/^(?<!%)\%\{?set_build_flags\b\}?//;
		s/(?<!%)\%\{?build_cflags\b\}?/%{optflags}/;
		    });
	    if ($sec_has_qmake) {
		my $qtver=3;
		$qtver=4 if $mainsec->match_body('^BuildReq.+qt4-devel');
		$qtver=5 if $mainsec->match_body('^BuildReq.+qt5-devel');
		$sec->subst_body(qr'^\s*qmake(?=(?:\s|$))','qmake-qt'.$qtver);
	    }
	} elsif ($t eq 'install') {
	    my $desktop_install=0;
	    $sec->map_body(sub {
		# to ALT macros
		s/(?<!%)\%make_install/%makeinstall_std/g;
		# exclude
		$_='# '.$_ if /^\%\{_fixperms\}/;
		#ldconfig -v -n  %{buildroot}%{_libdir}
		$_='# '.$_ if /^ldconfig\s.*\%\{_libdir\}/;

		# %fdupes - suse based - just in case
		s,(?<!%)\%\{?fdupes\}?\b,fdupes,;

		# desktop-file-install
		if (/^\s*desktop-file-install\s/ or ($desktop_install and /\\\n$/s)) {
		    $desktop_install=1;
		    $_='' if /^\s*--add-category\s+X-Fedora\s*\\\n/s;
		    # TODO: vendor
		    # 1) -> replace --vendor=fedora with %add_desktop_vendor
		    # 2) drop fedora-* in source / files / or detect applications/*
		    # if 2) fails, define add_desktop_vendor --vendor=fedora

		    # pathfix.py hack
		    #s!^(\s*)pathfix.py!$1%python3_libdir/Tools/pathfix.py!;

		} else {
		    $desktop_install=0;
		}

			   });

	    # move doc dir for > f19
	    $sec->multi_exclude_body(
		qr/^mv -f \$RPM_BUILD_ROOT\%\{_docdir\}\/\%\{name\}-\%\{version\} \\/,
		qr/\$RPM_BUILD_ROOT\%\{?_pkgdocdir/);

	} elsif ($t eq 'triggerun') {
	    if ($sec->match_body(qr'(?:chkconfig|systemctl)')) {
		$sec->delete;
		next;
	    }

	} elsif ($t eq 'post') {
	$sec->map_body(sub {
	    $flag_need_fedora_compat=1 if /%systemd_post/;
		       } );

	} elsif ($t eq 'postun') {
	$sec->map_body(sub {
	    $flag_need_fedora_compat=1 if /%systemd_postun(?:_with_restart)?\b/;
		       } );

	} elsif ($t eq 'files') {
	    #
	}

	$sec->map_body(sub {
	    s,\%\{_configure\},%{_configure_script},g; # found in RuFedora monodevel-java
	    if (s/(?<!%)\%\{?cmake3?\}?(?=\s|$)/%{fedora_cmake}/g) {
		$flag_need_fedora_compat=1;
		$flag_need_cmake=1;
	    }
	    $flag_need_cmake=1 if s,\%\{__cmake\},cmake,g;
	    $configure_macro_is_used=1 if /\%\{?configure/;
	    $flag_need_fedora_compat=1 if /(?<!%)\%\{?py[23]_shbang_opts\b\}?/;

	    #cp -f /usr/lib/rpm/config.{guess,sub} config/
	    $_='# '.$_ if m!^\s*cp\s+-[fp]\s+/usr/lib/rpm/(?:redhat/|config\.)!;
	    # cp /usr/lib/rpm/redhat/config.{guess,sub} .
	    s,^\s*cp\s+/usr/lib/rpm/redhat/,# cp /usr/lib/rpm/redhat/,;
	    #cp /usr/lib/rpm/config.guess config.guess
	    s,^\s*cp\s+/usr/lib/rpm/config,# cp /usr/lib/rpm/config,;

	} ) if ($t eq 'install' or $t eq 'build' or $t eq 'prep' or $t eq 'check');

	# general hacks here
	$sec->map_body(sub {

	    warn "WARNING: Fedora specific docbook: $_" if m![\}/]sgml/docbook/!;

	    $flag_need_fedora_compat=1 if /\%\{(?:_jsdir|_rundir|build_ldflags)\}/;
	    $flag_need_fedora_compat=1 if /\%\{?\??__(?:isa_bits|global_ldflags)\}?/;
	    $flag_need_fedora_compat=1 if /(?<!%)\%\{?_metainfodir\b\}?/;

	    # contains section, so they can be anywhere
	    s,\%\{?ldconfig_scriptlets\b\}?(?:\s+-n)?(?:\s+\S+)?,,;

	    # X11 app-defaults; todo: move to Generic, as this is alt-specific, not fedora-specific
	    s,\%\{?_datadir\}?/X11/app-defaults,%{_x11appconfdir},g;
	    s,\%\{?_datadir\}?(/X11/[^/]+/app-defaults/),%{_sysconfdir}$1,g;
	    # subst first dir w/o app-defaults:
	    #%dir %{_datadir}/X11/ja/
	    #%dir %{_datadir}/X11/ja/app-defaults/
	    s,\%\{?_datadir\}?(/X11/(?:ja|[a-zA-Z][a-zA-Z]|iso8[^/]+|koi[^/]+|[^/]+\.UTF-8)/?),%{_sysconfdir}$1,g;

	    # dislocations
	    s,\%\{_sbindir\}/strfile,%{_bindir}/strfile,g;

	    # translate macros first, as they are used in complex constructs
	    s,\%\{_pkgdocdir\},%{_docdir}/%{name},g; #-%{version} ? better not, see tuxmath
	    s,\%_pkgdocdir\b,%_docdir/%{name},g; #-%{version} ?

	    if (/(?<!%)\%\{?_texmf_main\}?/) {
		s,\%\{_texmf_main\},%{_texmfmain},g;
		s,\%_texmf_main,%_texmfmain,g;
		$flag_macro_texmf=1;
	    }

	    if (/(?<!%)\%\{?_kde4_(?:prefix|iconsdir|sbindir|bindir)\}?/) {
		s,\%\{_kde4_prefix\},%{_K4prefix},g;
		s,\%_kde4_prefix,%_K4prefix,g;
		s,\%\{_kde4_iconsdir\},%{_K4iconsdir},g;
		s,\%_kde4_iconsdir,%_K4iconsdir,g;
		s,\%\{_kde4_sbindir\},%{_K4sbindir},g;
		s,\%_kde4_sbindir,%_K4sbindir,g;
		s,\%\{_kde4_bindir\},%{_K4bindir},g;
		s,\%_kde4_bindir,%_K4bindir,g;
		$flag_need_fedora_compat=1;
		$flag_macro_kde=1;
	    }

	    if (/(?<!%)\%\{?_kf5_/) {
		$flag_need_fedora_compat=1;
		$flag_macro_kf5=1;
	    }

	    $flag_macro_erlang=1 if s,\%erlangdir,%_otpdir,g;
	    $flag_macro_erlang=1 if s,\%\{erlangdir\},%{_otpdir},g;
	    $flag_macro_fedoraver=1 if m'(?<!%)\%\{\??fedora[:}]';
	    $flag_macro_fedoraver=1 if m'(?<!%)\%\??fedora(?:[\s=<>]|$)';
	    $flag_macro_nodejs=1 if m'(?<!%)\%\{?nodejs_site(?:lib|arch)';

	    $flag_need_fedora_compat=1 if (
		m'(?<!%)\%\{?(?:ocaml|mono|ghc|GNAT|GPRbuild|openblas)_arches\}?' or
		# qt4 compat macroses
		m'(?<!%)\%\{?_qt4[}_]' or
		# perl compat
		m'(?<!%)\%\{?fix_shbang_line\}?' or
		m/(?<!%)\%\{?perl_testdir\}?/ or
		# kde4 compat
		m'(?<!%)\%\{?(?:cmake_kde4|_kde4_[\w\d_]+)\}?' or
		# sugar
		m'(?<!%)\%\{?sugar(?:activity|lib)dir\}?' or
		# misc
		m'(?<!%)\%\{?_(?:datarootdir|_id_u)\}?(?=(?:\W|$))' or
		m'(?<!%)\%\{?_cups_serverbin\}?'
	    );

	    $flag_macro_emacs=1 if s,\%\{_emacs_sitelispdir\},%{_emacslispdir},g;
	    $flag_macro_emacs=1 if s,\%_emacs_sitelispdir,%_emacslispdir,g;
	    $flag_macro_emacs=1 if s,\%\{_emacs_sitestartdir\},%{_emacs_sitestart_dir},g;
	    $flag_macro_emacs=1 if s,\%_emacs_sitestartdir,%_emacs_sitestart_dir,g;
	    # not %{...}! rpm404 feature: %foo arg, not %{foo} arg
	    $flag_macro_emacs=1 if s,\%\{_emacs_bytecompile\},%byte_compile_file,g;
	    $flag_macro_emacs=1 if s,\%_emacs_bytecompile,%byte_compile_file,g;

	    s,\%\{_xemacs_sitelispdir\},%{_xemacs_lispdir},g;
	    s,\%_xemacs_sitelispdir,%_xemacs_lispdir,g;
	    # not %{...}! rpm404 feature: %foo arg, not %{foo} arg
	    s,\%\{_xemacs_bytecompile\},%xemacs_bytecompile,g;
	    s,\%_xemacs_bytecompile,%xemacs_bytecompile,g;

	    s,\%\{_tmppath\},%{_var}/tmp,g;
	    s,\%\{_systemd_unitdir\},%{_unitdir},g;
	    s,\%\{_prefix\}/lib/tmpfiles.d,%{_tmpfilesdir},g;
	    s,\%\{_rpmconfigdir\}/macros.d,%{_rpmmacrosdir},g;
	    s,\%\{rpmmacrodir\},%{_rpmmacrosdir},g;

	    $flag_macro_go=1 if s,\%\{gopath\},%{go_path},g;
	    $flag_macro_go=1 if s,\%gopath,%go_path,g;
	    $flag_macro_php=1 if s,\%\{php_extdir\},%{php7_extdir},g;

	    # translate complex constructs last

	    return if s,^\%\{\!?\??(?:perl_default_filter|filter_setup):(?=\s*$),%{echo ,;
	    return if s,^\%if\s*\%\{\!?\??(?:perl_default_filter|filter_setup)\b,%if 0%{echo,;
	    # brackets {} balance is important, as closing bracket for %{echo above can be %?perl_default_filter}
	    s,(?<!%)\%\{\!?\??(?:perl_default_filter|filter_setup)\},,;
	    s,(?<!%)\%\!?\??(?:perl_default_filter|filter_setup)\b,,;

	    # kill fancy fedora hacks that may disrupt alt mashinery
	    s/^\%(global\s+(?:_use_internal_dependency_generator|__deploop|__find_provides|__find_requires))/#$1/;
	    s/^(\s*chmod\s+\S+\s+\%\{?(?:__find_provides|__find_requires))/#$1/;
	    if (/^\%filter_from_(?:provides|requires)/) {
		s!::!.!g if /perl\(/;
		s!perl\(([^\)]+)\)!perl.$1.pm.!g;
		# and glue it together like in %filter_from_requires /perl(...)$/d /perl(...)$/d
		s!/d\s+/!/d;/!g;
		$filter_from_provreq_is_used=1;
	    } elsif (s/^\%global\s+__(provides|requires)_exclude\s+(.*)$//) {
		if ($filter_from_provreq_is_used) {
		    # entirely skip the conversion (old rpm 4.9) in favor of filter_from_...
		} else {
		    #%global __provides_exclude %{?__provides_exclude}|perl\\(Experimental::Sample\\)|perl\\(fail_example\\)|perl\\(fail_example_testsuite_setup\\)
		    #%global __requires_exclude %{?__requires_exclude}|perl\\(Exporter\\)
		    #%global __requires_exclude %{?__requires_exclude:%__requires_exclude|}^perl\\((Encode|File::ShareDir|MIME::Words)\\)$
		    my $cmd='%filter_from_'.$1;
		    my $opts=$2;
		    my $arg='';
		    $opts=~s/\%\{?\??__(?:provides|requires)_exclude(?::\%\{?__(?:provides|requires)_exclude\}?)?\|?\}?\|?//g;
		    if ($opts=~/perl\\\\\(/) {
			$opts=~s!::!/!g;
			# group (A|B|C|..) to sed
			$opts=~s/(?<!\\)([(|)])/\\$1/g;
			$opts=~s!perl\\\\\(!perl(!;
			$opts=~s!\\\\\)\*?\$?$!.pm)!g;
			#$opts=~s/perl\\\\\(([\w_:]+)\\\\\)/&__perldep2exclpttern($1)/eg;
		    }
		    $opts=~s!/!.!g;
		    push @__push_mainsec, $cmd.' /'.$opts.'/d'."\n";
		}
	    } elsif (s/^\%filter_(prov|req)(?:ides|uires)_in\s+(.*)$//) {
		my $type=$1;
		my $path=$2;
		$path=~s/\s*$//;
		$path=~s!/$!/*!;
		push @__push_mainsec, '%add_find'.$type.'_skiplist '.$path."\n";
	    }

		       } ) if $t ne 'changelog';

    };

    unless ($configure_macro_is_used) {
	foreach my $sec ($spec->get_sections()) {
	    next if $sec->get_type ne 'files';
	    # Oops! did not work in case of arptables_jf
	    #$sec->subst_body(qr'\%{?_libexecdir}?/','/usr/libexec/');
	}
    }

    $mainsec->push_body(@__push_mainsec) if @__push_mainsec;
    my $buildreq=$resourceobj->buildreq;
    $buildreq->add_pre_buildreq('rpm-macros-fedora-compat') if $flag_need_fedora_compat;
    $buildreq->add_pre_buildreq('rpm-build-tex') if $flag_macro_texmf;
    $buildreq->add_pre_buildreq('rpm-macros-emacs') if $flag_macro_emacs;
    $buildreq->add_pre_buildreq('rpm-build-erlang') if $flag_macro_erlang;
    $buildreq->add_pre_buildreq('rpm-macros-kde-common-devel') if $flag_macro_kde;
    $buildreq->add_pre_buildreq('rpm-build-kf5') if $flag_macro_kde;
    $buildreq->add_pre_buildreq('rpm-macros-nodejs') if $flag_macro_nodejs;
    $buildreq->add_pre_buildreq('rpm-macros-golang') if $flag_macro_go;
    $buildreq->add_pre_buildreq('rpm-build-php7') if $flag_macro_php;
    $buildreq->add_buildreq('cmake') if $flag_need_cmake; # like kde apps ...
    $buildreq->add_buildreq('libqt4-devel') if $flag_qt4_devel;
    $buildreq->add_buildreq('qt5-base-devel') if $flag_qt5_devel;
    $buildreq->add_buildreq('rpm-build-golang') if $flag_macro_go;
    $mainsec->unshift_body('%define fedora '.$FEDORA_VERSION."\n") if $flag_macro_fedoraver;

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_fedora2alt_kill_python_duplicates {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $rename=$opt->{-rename};
    return RPM::Source::TransformContainer::TRANSFORMATION_OK
	if !$rename or $rename!~/^python-module-/;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	next unless $t eq 'package' or $t eq 'description';
	my $sec_pkg = $sec->get_raw_package();
	# skip main section
	next unless $sec_pkg;
	my $sec_pkg_expanded = $sec->get_package_name();
	next if $sec_pkg_expanded ne $rename;
	$sec->delete;
	print STDERR "info: dropped duplicate \%$t $sec_pkg_expanded\n" if $Source::Shared::verbose>1 or $verbose;
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;

# TODO: share?
# note: do not merge with other filers. Tag range include Conflicts/Provides/etc.
package RPM::Source::Tools::Dependency::Filter::MacrosCleanup;
use strict;
use warnings;
our $verbose=0;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $name=$dep->{-name};
    #print STDERR "DEBUG:Filter::MacrosCleanup: name=$name\n" if $verbose;
    $name=~ s/\(?\%\{?\??__?isa(?:$|\})\)?//;
    $name=~ s/\%\{python2_pkgprefix\}-devel/python-devel/;
    #print STDERR "DEBUG:Filter::MacrosCleanup: newname=$name\n" if $verbose and $name ne $dep->{-name};
    $dep->{-name}=$name;
    return $dep;
}

package RPM::Source::Tools::Dependency::Filter::FedoraMapSpecificDeps;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $name=$dep->{-name};
    $name=~ s/\%\{?\??_isa(?:$|\})//;
    $name='%{_sysconfdir}/httpd2/conf' if $name eq '%{_sysconfdir}/httpd/conf.d';
    $name='perl' if $name eq 'perl(:VERSION.pm)';
    $dep->{-name}=$name;
    return $dep;
}

package RPM::Source::Tools::Dependency::Filter::FedoraDropNotImplementedDeps;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;
our $verbose=1;
sub apply {
    my ($self, $tagobj, $dep) = @_;
    if ($dep->{-name} =~ /^mimehandler\(/) {
	print STDERR "info: dropped dependency $dep->{-name} - not yet supported in ALT \n" if $verbose;
	delete $dep->{-name};
    } elsif ($dep->{-name} =~ /^kmod\(/) {
	print STDERR "info: dropped dependency $dep->{-name} - not yet supported in ALT \n" if $verbose;
	delete $dep->{-name};
    };
    return $dep;
}

1;

__END__
