package RPM::Source::Convert::Plugins::FontUsage2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory::DependencyFilter;
use RPM::Source::Transformation::OptHandlerWrapper;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'FontPath dependencies',
    -description => 'convert generic (Fedora-like) font paths to alt in dependencies',
    -class => 'RPM::Source::Dependency::Filter::FontUsage2ALT',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS | SPEC_TAG_PROVIDES,
    -priority => FILTER_PRIORITY_DEFAULT,
);

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'font_usage_2alt_convert',
	    -description => 'convert generic (Fedora-like) font paths to alt in spec',
	    -opt_handler_sub=>\&opt_handler_font_usage2alt_convert),
}

sub opt_handler_font_usage2alt_convert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	$sec->map_body(sub {
	    if (m!\.ttf! or m!/usr/share/fonts/!) {
		$_=&___::FontUsage2ALT::fontpath2alt($_);
	    }
	}) if ($t eq 'install');
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

package ___::FontUsage2ALT;
use strict;
use warnings;
sub fontpath2alt {
    my ($in)=@_;
    # manaplus, for example
    $in=~s,/fonts/liberation\b,/fonts/ttf/liberation,;
    # python-pgu
    $in=~s,/fonts/bitstream-vera\b,/fonts/ttf/TrueType-vera,;
    # xplanet widelands rt TnL-data
    $in=~s,/fonts/(amiri|dejavu|gnu-free|google-droid|lklug|mplus|wqy-microhei)\b,/fonts/ttf/$1,;
    return $in;
}

package RPM::Source::Dependency::Filter::FontUsage2ALT;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $name=$dep->{-name};
    if ($name=~/^(\%|\/).*\.ttf$/) {
	$dep->{-name}=&___::FontUsage2ALT::fontpath2alt($name);
    }
    return $dep;
}

1;

__END__
