package RPM::Source::Convert::Plugins::PerlAutodepsUseStyle2PathStyle;
use strict;
use warnings;
#use Carp;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Factory::DependencyFilter;
our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

$RPM::Source::Tools::Dependency::Tag::MACROS_TAG{'add_both'} = 
    RPM::Source::Tools::Dependency::Filter::SPEC_TAG_REQUIRES |
    RPM::Source::Tools::Dependency::Filter::SPEC_TAG_BUILD_REQUIRES;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Perl UseStyle to PathStyle dependency filter',
    -description => 'convert Perl UseStyle dependencies to PathStyle',
    -class => 'Perl::DependencyFilter::PerlAutodepsUseStyle2PathStyle',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS | SPEC_TAG_PROVIDES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'perl_autodeps_use_style_to_path_style',
	    -opt_handler_sub=>\&opt_handler_perlstyle_use2path_convert);
}

sub opt_handler_perlstyle_use2path_convert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    my $mainsec=$spec->main_section;
    my $buildarch=$mainsec->get_tag('BuildArch');
    my $is_main_noarch=1 if $buildarch and $buildarch eq 'noarch';
    
    my ($flag_macro_perl,$flag_catalyst_pl,$flag_module_build);
    my ($filter_from_provreq_is_used,$perl_buildsystem_is_used);
    my @__push_mainsec;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	next if $t eq 'changelog';
	if ($t eq 'package') {
	    # we use filter
	} elsif ($t eq 'prep') {
	    $sec->map_body(sub {
		$flag_catalyst_pl=1 if /catalyst.pl(?:\s|$)/;
			   });

	} elsif ($t eq 'build') {
	    $sec->map_body(sub {
		if (/^(.*)Makefile.PL\s+/) {
		    my $precmd=$1;
		    # %{__perl} Makefile.PL INSTALLDIRS=vendor|perl
		    if ($precmd=~/perl/) {
			#s/Makefile.PL\s+/Makefile.PL INSTALLMAN1DIR=%_man1dir /;
			$perl_buildsystem_is_used=1;
		    }
		} elsif (/^(.*)Build.PL\s+/) {
		    my $precmd=$1;
		    if ($precmd=~/perl/) {
			# %{__perl} Build.PL installdirs=vendor --install_path bindoc=%_man1dir
			#s/Build.PL\s+/Build.PL --install_path bindoc=%_man1dir /;
			$perl_buildsystem_is_used=1;
			#Module/Build(/Tiny).pm is required
			$flag_module_build=1;
		    }
		}
			   } );

	} elsif ($t eq 'files') {
	    #
	}

	# general hacks here
	$sec->map_body(sub {
	    if (/perl_(?:arch|priv|vendor|site)(?:arch|lib)/) {
		s,%\{perl_archlib\},%{perl_vendor_archlib},g;
		s,%perl_archlib,%perl_vendor_archlib,g;
		s,%\{perl_vendorarch\},%{perl_vendor_archlib},g;
		s,%perl_vendorarch,%perl_vendor_archlib,g;
		s,%\{perl_sitearch\},%{perl_vendor_archlib},g;
		s,%perl_sitearch,%perl_vendor_archlib,g;
		s,%\{perl_sitelib\},%{perl_vendor_archlib},g;
		s,%perl_sitelib,%perl_vendor_archlib,g;
		s,%compat_perl_vendorarch,%perl_vendor_archlib,g;
		s,%\{compat_perl_vendorarch\},%{perl_vendor_archlib},g;
		s,%\{perl_privlib\},%{perl_vendor_privlib},g;
		s,%perl_privlib,%perl_vendor_privlib,g;
		if (
		    $DistroMap::default_originrepo ne 'mageia'
		    or $is_main_noarch) {
		    s,%\{perl_vendorlib\},%{perl_vendor_privlib},g;
		    s,%perl_vendorlib,%perl_vendor_privlib,g;
		} else {
		    # mageia arch TODO
		    s,%\{perl_vendorlib\},%{perl_vendor_archlib},g;
		    s,%perl_vendorlib,%perl_vendor_archlib,g;
		}
		$flag_macro_perl=1;
	    }
	} );
    }

    if ($perl_buildsystem_is_used) {
	foreach my $sec ($spec->get_sections()) {
	    my $type = $sec->get_type();
	    if ($type eq 'files') {
		# %exclude %{_mandir}/man3/DateTime.*
		$sec->exclude_body(qr'^\s*(\%exclude\s+)?\%\{_mandir\}/man3/');
		#%{_mandir}/man?/Net::Server::Coro*.*
		$sec->exclude_body(qr'^\s*(\%exclude\s+)?\%\{_mandir\}/man\?/\w+::');
	    }
	}
    }

    $mainsec->push_body(@__push_mainsec) if @__push_mainsec;
    my $buildreq=$resourceobj->buildreq;
    $buildreq->add_pre_buildreq('rpm-build-perl') if $flag_macro_perl;
    $buildreq->add_buildreq_for('used in spec file','/usr/bin/catalyst.pl') if $flag_catalyst_pl;
    # can be Module-Build-Tiny
    #$buildreq->add_buildreq_for('used in spec file','perl-Module-Build') if $flag_module_build;
    # TODO: implement if $resourceobj -> lang -> perl
    $buildreq->add_buildreq('perl-podlators') if $flag_macro_perl or $buildreq->has_buildreq('perl-devel');

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

package Perl::DependencyFilter::PerlAutodepsUseStyle2PathStyle;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub __convert2alt_fedora_perl_provides {
    my $prov=shift;
    #my $old=$prov;
    $prov=~s!::!/!g if $prov=~/perl\(/;
    $prov=~s!perl\(([^\)]+)\)!perl($1.pm)!g;
    #print STDERR "CONVERTED: $old -> $prov\n";
    return $prov;
}

sub __convert2alt_fedora_perl_version {
    my $fcperlver=shift;
    $fcperlver=~s/^(\d+\.\d\d\d)(\d\d\d)$/$1.${2}/ or
    $fcperlver=~s/^(\d+\.\d\d\d)(\d\d)$/$1.${2}0/ or
    $fcperlver=~s/^(\d+\.\d\d\d)(\d)$/$1.${2}00/ or
    $fcperlver=~s/^(\d+\.\d\d)$/${1}0/ or
    $fcperlver=~s/^(\d+\.\d)$/${1}00/;
    $fcperlver=~s/\.000\.0+$/.0/;
    $fcperlver=~s/\.000$/.0/;
    return $fcperlver;
}

# https://bugzilla.altlinux.org/show_bug.cgi?id=27367
my $apt_alt_bug=1;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    return $dep if $pkgname!~/^perl\((.+)\)$/;
    my $perlreq=$1;
    return $dep if $perlreq=~/\.p(?:h|l|m)$/;
    # looks like fedora perl() provides
    $perlreq=~s!::!/!g;
    $perlreq.='.pm';
    $dep->{-name}='perl('.$perlreq.')';
    my $perlver=$dep->{-version};
    if ($perlver) {
	if ($apt_alt_bug && ($tagobj->tagbits & RPM::Source::Tools::Dependency::Filter::SPEC_TAG_BUILD_REQUIRES)) {
	    # TMP HACK TO DELETE!!! til https://bugzilla.altlinux.org/show_bug.cgi?id=27367
	    $dep->{-version}=undef;
	} else { 
	    $dep->{-version}=&__convert2alt_fedora_perl_version($perlver);
	}
    }
    return $dep;
}

1;
__END__
