package RPM::Source::Convert::Plugins::Python;
use strict;
use warnings;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Factory::DependencyFilter;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Python PreDistromap',
    -description => 'preconvert python macro in Python package names before distromap',
    -class => 'RPM::Source::Tools::Dependency::Filter::PythonPreDistromap',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Python PostDistromap',
    -description => 'convert generic Python name conventions to alt name conventions',
    -class => 'RPM::Source::Tools::Dependency::Filter::PythonPostDistromap',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS,
    -priority => FILTER_PRIORITY_POSTDISTROMAP,
);


sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'generic_python_related_preconvert',
	    -opt_handler_sub=>\&opt_handler_python_preconvert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'generic_python_related_convert',
	    -opt_handler_sub=>\&opt_handler_python_convert),
	;
}

sub opt_handler_python_preconvert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    # for proper conversion of subpackage names
    $macros->set_raw('python3_pkgversion',3);
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_python_convert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    my $mainsec=$spec->main_section;
    my $buildreq=$resourceobj->buildreq;

    my ($flag_macro_python2,$flag_macro_python3);
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	next if $t eq 'changelog';
	if ($t eq 'build') {
	    $sec->map_body(sub {
		# warning: setuptools is a hack, remove me when fixed in SourceAnalyser
		$buildreq->add_buildreq('python-devel','python-module-setuptools') 
		    if s/(?<!%)\%\{?py2?_build\b\}?/%python_build/g;
		$buildreq->add_buildreq('python3-devel','python3-module-setuptools') 
		    if s/(?<!%)\%\{?py3_build\b\}?/%python3_build/g;
			   });
	} elsif ($t eq 'install') {
	    $sec->map_body(sub {
		s/(?<!%)\%\{?py2?_install\b\}?/%python_install/g;
		s/(?<!%)\%\{?py3_install\b\}?/%python3_install/g;

		# HACK ... not exactly a direct translation ... and does not work
		# Byte compile the python files
		#%py_byte_compile %{__python} $RPM_BUILD_ROOT%{pkgdir}/xemacs-packages/etc/python-modes
		#$flag_macro_python2=1 if s/^\s*\%py_byte_compile\s+\%\{__python2?\}\s+\$RPM_BUILD_ROOT/%add_python_compile_include /;
			   });

	} elsif ($t eq 'files') {
	    #
	    $sec->map_body(sub {
		# python3 packaging (in Fedora)
		s,\.cpython-\%\{python3_version_nodots\}m(?:-\*)?\.so\b,%{_python3_extension_suffix},;
		# python3 packaging TMP HACK!!! TODO!!!
		s,\.cpython-\%\{python3_version_nodots\}\.,.cpython-*.,;
		s,\.cpython-\%\{python3_version_nodots\}\*,.cpython-3*,;
		s,\%\{python3_version_nodots\}\*,3*,;
			   } );

	}

#	$sec->map_body(sub {
#	} ) if ($t eq 'install' or $t eq 'build' or $t eq 'prep' or $t eq 'check');



	# general hacks here
	$sec->map_body(sub {

	    if (/(?<!%)\%\{?python2?_site(?:lib|arch)\}?(?!dir)/) {
		s,\%\{python2?_sitelib\},%{python_sitelibdir_noarch},g;
		s,\%python2?_sitelib,%python_sitelibdir_noarch,g;
		s,\%\{python2?_sitearch\},%{python_sitelibdir},g;
		s,\%python2?_sitearch,%python_sitelibdir,g;
		$flag_macro_python2=1;
	    }
	    if (/(?<!%)\%\{?__python2\}?/) {
		s,\%\{__python2\},%{__python},g;
		s,\%__python2(\s),%{__python}$1,g;
		$flag_macro_python2=1;
	    }

	    if (/(?<!%)\%\{?python3_site(?:lib|arch)\}?(?!dir)/) {
		s,(?<!%)\%\{python3_sitelib\},%{python3_sitelibdir_noarch},g;
		s,(?<!%)\%python3_sitelib,%python3_sitelibdir_noarch,g;
		s,(?<!%)\%\{python3_sitearch\},%{python3_sitelibdir},g;
		s,(?<!%)\%python3_sitearch,%python3_sitelibdir,g;
		$flag_macro_python3=1;
	    }
	    if (
		s,(?<!%)\%\{py3dir\},%{_builddir}/python3-%{name}-%{version}-%{release},g or
		s,(?<!%)\%py3dir,%{_builddir}/python3-%{name}-%{version}-%{release},g) {
		$flag_macro_python3=1;
	    }
	    $flag_macro_python3=1 if m'(?<!%)\%\{?__python3\}?';

	    s,(?<!%)\%\{python_version\},%{__python_version},g if not $macros->get_raw('python_version');
	    s,(?<!%)\%\{python2_version\},%{__python_version},g if not $macros->get_raw('python2_version');
	    s,(?<!%)\%\{python3_version\},%{__python3_version},g if not $macros->get_raw('python3_version');

			   } );

    }
    if ($flag_macro_python2) {
	$buildreq->add_pre_buildreq('rpm-build-python');
	$buildreq->add_buildreq('python-devel');
    }
    if ($flag_macro_python3) {
	$buildreq->add_pre_buildreq('rpm-build-python3');
	$buildreq->add_buildreq('python3-devel');
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

package RPM::Source::Tools::Dependency::Filter::PythonPreDistromap;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $macros=$self->{-spec}->macros;
    my $pkgname=$dep->{-name};
    # noticed in fedora / epel
    $pkgname=~s,\%\{python3_pkgversion\},3,g;
    $dep->{-name}=$pkgname;
    return $dep;
}

package RPM::Source::Tools::Dependency::Filter::PythonPostDistromap;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    if ($pkgname=~/^python3?-(?:base$|contextkit$|dev$|devel$|test$|tools|module-)/) {
	# skip below branch
    } elsif ($pkgname=~s/^python2-/python-module-/) {
	$dep->{-name}=$pkgname;
    } elsif ($pkgname=~s/^python3-/python3-module-/) {
	$dep->{-name}=$pkgname;
    }
    return $dep;
}

1;
