package RPM::Source::Convert::Plugins::SetDocNoarch;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=1;

use constant ARCH_SURE => 2;
use constant ARCH_POSSIBLE => 1;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'font_usage_2alt_convert',
	    -description => 'check doc packages to be noarch',
	    -opt_handler_sub=>\&opt_handler_set_doc_noarch),
}

sub opt_handler_set_doc_noarch {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my @arch_doc_sec;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	next if $t ne 'package';
	my $name=$sec->get_package_name;
	next unless $name=~/-(doc|docs|manual|javadoc)$/;
	my $is_arch=&__is_arch_package($sec);
	push @arch_doc_sec, $sec if $is_arch;
	print STDERR "DEBUG: $name is arch docs:\n", $sec->describe() if $is_arch and $verbose>3;
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK unless @arch_doc_sec;

    my $is_main_arch=&__is_arch_package($spec->main_section);
    #return RPM::Source::TransformContainer::TRANSFORMATION_OK if not $is_main_arch;

    foreach my $sec (@arch_doc_sec) {
	my $rname=$sec->get_raw_package;
	my $filessec=$spec->get_section('files',$rname);
	unless ($filessec) {
	    print STDERR "INFO: $rname looks like arch docs, but no matching %files found.\n"
		if $is_main_arch and $verbose;
	} else {
	    if (&__is_arch_files($filessec)==ARCH_SURE) {
		print STDERR "INFO: \%package $rname looks like docs but is arch package.\n";
		    $filessec->describe() if $verbose >1;
	    } elsif (&__is_arch_files($filessec)==ARCH_POSSIBLE) {
		print STDERR "INFO: \%package $rname looks like docs. Should be noarch. %files:\n",
		    $filessec->describe() if $verbose;
	    } elsif (!&__is_arch_files($filessec)) {
		print STDERR "INFO: \%package $rname looks like docs. Set as noarch.\n" if $is_main_arch and $verbose;
		$sec->push_body('BuildArch: noarch'."\n");
	    }
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __is_arch_package {
    my $sec=shift;
    my $arch=1;
    $sec->visit_body(sub {
	if (m!^BuildArch\S*:\s+noarch\s*$!i) {
	    $arch=0;
	}
		     });
    return $arch;
}

sub __is_arch_files {
    my ($filessec)=@_;
    my $arch=0;
    $filessec->visit_body(sub {
	s,^\%attr\s*\S+\s*,,;
	unless (
	    m!^\s*(?:\#|$)! or
	    m!^\s*\%if\s! or
	    m!^\s*\%(?:else|endif)! or
	    m!^\s*\%doc\s! or
	    m!^\s*\%license\s! or
	    #%{!?_licensedir: %doc %{_pkgdocdir}/LICENSE}
	    #%{?_licensedir: %license LICENSE}
	    m!^\%\{\!?\?_licensedir:\s*\%(?:doc|license)\b! or
	    m!^\s*\%defattr! or
	    m!^\s*(?:\%dir\s+)?\%\{?_datadir\}?/! or
	    m!^\s*(?:\%dir\s+)?\%\{?_docdir\}?/! or
	    m!^\s*(?:\%dir\s+)?\%\{?_defaultdocdir\}?/! or
	    m!^\s*(?:\%dir\s+)?\%__distdoc/! or
	    m!^\s*(?:\%dir\s+)?\%\{?(?:our_docdir|_pkgdocdir|_javadocdir)\}?!
	    ) {
	    #print STDERR "DEBUG: __is_arch_files: archline=$_";
	    $arch=ARCH_POSSIBLE;
	    if (
		m!%\{?_libdir\b! or
		m!%\{?tde_libdir\b!
		) {
		$arch=ARCH_SURE;
	    }
	}
    });
    #print STDERR "DEBUG: __is_arch_files: arch=$arch\n", $filessec->describe();
    return $arch;
}


1;

__END__
