package RPM::Source::Convert::Plugins::SuSE2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Convert::Plugins::Generic2ALT;
use RPM::Source::Transformation::Factory::Buildreq;
use Source::Shared::Resource::DistroMap;
use RPM::Source::Convert::Plugins::PerlAutodepsUseStyle2PathStyle;
use RPM::Source::Convert::Plugins::Python;
use RPM::Source::Convert::Plugins::OtherLang;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

$DistroMap::default_originrepo='suse';
$DistroMap::default_originbranch=undef;

my $SUSE_VERSION=1550;

# fake section to be processed separately
$RPM::Source::SpecSection::SECTION_MACROS{'verifyscript'}=1;
$RPM::Source::SpecSection::SECTION_MACROS{'debug_package'}=1;

## TODO: drop!!! and replace with proper filter!!!
use RPM::Source::Convert::Plugins::DistroMap::Packages;

map {$RPM::Source::Convert::Plugins::DistroMap::Packages::DEP_SKIP{$_} = 1} # SUSE specific PreReq:
qw!%fillup_prereq %insserv_prereq!;

my %DEP_SUBST = (qw/update-desktop-files rpm-build-suse-compat/,
' ' => ' ',
);
foreach my $key (keys(%DEP_SUBST)) {
    $RPM::Source::Convert::Plugins::DistroMap::Packages::DEP_SUBST{$key} = $DEP_SUBST{$key};
}


# see also http://ru.opensuse.org/Сборка_пакетов/Соглашение_по_стилю_RPM-пакетов/Макросы_RPM

# TODO
#%build
#%{__make} %{?jobs:-j%jobs} %{?_smp_mflags}

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'suse2alt_convert',
	    -opt_handler_sub=>\&opt_handler_suse2alt_convert);
}

sub opt_handler_suse2alt_convert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $mainsec=$spec->main_section;
    my ($flag_macro_suse,$flag_need_suse_compat, $flag_need_suse_build, $flag_need_cmake,
	$flag_need_qt5);
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	next if $t eq 'changelog';
	if ($t eq 'package') {
	    $sec->exclude_body(qr'^\s*Autoreqprov: on');
	} elsif ($t eq 'build') {
	} elsif ($t eq 'verifyscript' or $t eq 'debug_package') {
	    $sec->delete();
	    next;
	} elsif ($t eq 'install') {
	$sec->map_body(sub {
	    $flag_need_suse_build=1 if m/(?<!%)\%\{?suse_update_desktop_file\}?/;
	    # /var/adm/fillup-templates -> /etc/sysconfig
	    s,/var/adm/fillup-templates(?=(?:\s|$)),/etc/sysconfig,;
	    s,/var/adm/fillup-templates/sysconfig\.,/etc/sysconfig/,;
	    # fdupes -
	    s,(?<!%)\%\{?fdupes\}?\b,fdupes,;
		       });
	} elsif ($RPM::Source::SpecConst::SCRIPT_SECTION{$t}) {
	    $sec->exclude_body(qr'(?<!%)\%\{?(?:insserv_cleanup|insserv_force_if_yast|fillup_only|fillup_and_insserv|install_info_delete|install_info|run_ldconfig|set_permission)\}?');
	} elsif ($t eq 'files') {
	$sec->map_body(sub {
	    # /var/adm/fillup-templates -> /etc/sysconfig
	    s,/var/adm/fillup-templates/sysconfig\.,/etc/sysconfig/,;
	    s,/var/adm/fillup-templates/\*,/etc/sysconfig/*,;

	    #%{_mandir}/man1/mjpg_streamer.1%{ext_man}
	    s,\%\{?ext_man\}?(\s*)$,*$1,;

		       } );

	}


	# general hacks here
	$sec->map_body(sub {
	    $flag_macro_suse=1 if /(?<!%)\%\{\??suse_version\}/;
	    $flag_need_suse_compat=1 if /(?<!%)\%\{?\??(?:lang_package)\b/;

	    # macros.cmake
	    if (s/(?<!%)\%\{?cmake\}?(?=\s|$)/%{suse_cmake}/g or
		s/(?<!%)\%\{?make_jobs\}?(?=\s|$)/%{suse_make_jobs}/g or
		s/(?<!%)\%\{?cmake_install\}?(?=\s|$)/%{suse_cmake_install}/g or
		s/(?<!%)\%\{?ctest\}?(?=\s|$)/%{suse_ctest}/g
		) {
		$flag_need_suse_compat=1;
		$flag_need_cmake=1;
	    }
	    $flag_need_cmake=1 if s,\%\{__cmake\},cmake,g;
	    $flag_need_cmake=1 if s,\%\{__ctest\},ctest,g;
	    s,(?<!%)(\%\{?__)(builddir|builder|sourcedir)(\}?)(?=\s|$),${1}_suse$2$3,g;
	    s,(?<!%)(\%define\s+__)(builddir|builder|sourcedir)(?=\s|$),${1}_suse$2,g;
	    # end macros.cmake

	    $flag_need_qt5=1 if s,(?<!%)(\%\{?)_libqt5_includedir,($1)_qt5_headerdir,g;
	    $flag_need_qt5=1 if s,(?<!%)(\%\{?)_libqt5_,($1)_qt5_,g;
	    $flag_need_qt5=1 if s,(?<!%)\%qmake5_install,%install_qt5,g;
	    $flag_need_qt5=1 if s,(?<!%)\%qmake5,%qmake_qt5,g;

	});

    }
    $mainsec->unshift_body('%define suse_version '.$SUSE_VERSION."\n") if $flag_macro_suse;

    my $buildreq=$resourceobj->buildreq;
    $buildreq->add_pre_buildreq('rpm-macros-qt5') if $flag_need_qt5;
    if ($flag_need_suse_build) {
	$buildreq->add_pre_buildreq('rpm-build-suse-compat');
    } elsif ($flag_need_suse_compat) {
	$buildreq->add_pre_buildreq('rpm-macros-suse-compat');
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;

__END__
