package RPM::Source::Convert::Plugins::rpm2rpm404;
use strict;
use warnings;
#use Carp;
use File::Basename;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::Factory::DependencyFilter;
use RPM::Source::Transformation::OptHandlerWrapper;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;
our $rpm404=1;
# come from Fedora2ALT
our $opt_rpm404_touch_ghosts=1;
# due to rpm404
#our $opt_force_ascii=1;
my $opt_force_ascii=$rpm404;

our @LONGOPT=(
    'rpm404!' => \$rpm404,
    'touch-ghosts!' => \$opt_rpm404_touch_ghosts,
#    'force-ascii!' => \$opt_force_ascii,
    );

sub print_longopt {
    print "rpm 2 rpm 4.0.4 conversion  options:\n";
    print "	--no-rpm404  disable conversion to rpm 4.0.4 altlinux format\n";
    print "	--no-touch-ghosts  disable touch in %install %ghost-s in %files\n";
#    print "	--no-force-ascii  do not drop non-ascii characters from description\n";
}

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
#	-rpm404 => 0,
	});
}

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'drop release in version',
    -description => 'drop release in version in Buildreq/Req tags, like Req: foo > 2.0-2',
    -class => 'rpm2rpm404::DependencyFilter::RPM404DropReleaseInVersion',
    -tags => SPEC_TAG_SET_REQUIRES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

sub transformations {
    return unless $rpm404;
    my @transformations=
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'convert to rpm404',
	    -opt_handler_sub=>\&opt_handler_rpm404_main);
    push @transformations,
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'rpm404_touch_ghosts',
	    -opt_handler_sub=>\&opt_handler_rpm404_touch_ghosts)
	if $opt_rpm404_touch_ghosts;
    return @transformations;
}

sub opt_handler_rpm404_main {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    &alt_generic_rpm404_check_ahead($spec,$spec->main_section);
    &alt_generic_rpm404_single_f_flag_in_files($spec);
    #my $mainsec=$spec->main_section;
    my $macros=$spec->macros;
    foreach my $section ($spec->get_sections()) {
	my $t = $section->get_type();
	print STDERR "rpm2rpm404: processing \%$t ",$section->get_raw_package,"\n" if $verbose>2;
	if ($t eq 'package') {
	    # encoding mismatch. alt assume 8 bit
	    # TODO: detect utf8?
	    #$section->exclude_body('^Summary\(de\):');
	    &alt_generic_cleanup_headers($section);
	} elsif ($t eq 'description') {
	    my $l=$section->get_flag('-l');
	    #$section->delete() if $l and $l=~/^(?:de|fr)$/;
	    $section->apply_body(
		sub {
		    my $line=shift;
		    # found in golang-github-influxdb-influxdb
		    # %{%description}
		    if ($line=~/^\%\{\%description\}/) {
			my $desc_body_ref=$spec->get_section('description','')->get_bodyref;
			my @description;
			my @in=@$desc_body_ref;
			shift @in; # kill header
			while (@in) {
			    my $desc_line=shift @in;
			    last if $desc_line=~/^\s*(?:#|$)/ and @description or $desc_line=~/^\s*\%(?:if|else|endif)/;
			    push @description, $desc_line;
			}
			return @description;
		    }
		    $line=&__kill_non_ascii($line) unless $l;
		    return $line;
		});
	} elsif ($t eq 'prep') {
		#$section->subst_body_if(qr'\s-F(?=\d)',' -F ',qr'^\s*\%patch(?:\d|\s)');
		$section->map_body(sub {
		    my $ii;
		    # rpm404 compat: glues line to %patch: %build
		    s,(?<=\s)-b(\S+),-b $1,g if /^\s*\%patch/;
		    # rpm404 compat
		    s,\%setup0\b,%setup,;

		    if (s/(?<!\%)\%apply_patches/#%%apply_patches/) {
			my $patchlist=$spec->raw_patchlist_ref;
			for (my $i=0; $i <@$patchlist;$i++) {
			    next unless $patchlist->[$i];
			    $_.='%patch'.$i.' -p1'."\n";
			}
		    }

		    if (s/^(.*)(?<!%)\%\{?auto(setup|patch)\}?\s*//) {
			my $autoprefix=$1;
			my $autotype=$2;
			my @args=Text::ParseWords::shellwords($_);
			my @setupargs;
			my $patcharg_p=' -p1';
			my $prev_arg='';
			while (@args) {
			    my $arg=shift @args;
			    if ($arg eq '-n' or $arg eq '-a' or $arg eq '-b') {
				push @setupargs, $arg, shift @args;
			    } elsif ($arg =~ s/^-S//) {
				$arg=shift @args if $arg eq '';
				if ($arg eq 'git_am' or $arg eq 'git') {
				    $patcharg_p=' -p1';
				} else {
				    warn "Generic2ALT: INFO: autosetup option -S $arg: ignored.\n";
				}
			    } elsif ($arg eq '-p') {
				$patcharg_p=' -p'.(shift @args);
			    } elsif ($arg =~ s/^-p//) {
				$patcharg_p=' -p'.$arg;
			    } elsif ($arg =~ s/^-b//) {
				; # IGNORE for now
			    } elsif ($arg eq '-D' or $arg eq '-T' or $arg eq '-q' or $arg eq '-c') {
				push @setupargs, $arg;
			    } elsif ($prev_arg eq '-c') {
				push @setupargs, '-n', $arg;
			    } else {
				push @setupargs, $arg;
			    }
			    $prev_arg=$arg;
			}
			my $PCHREF=$spec->raw_patchlist_ref;
			my @PCHNUM;
			for ($ii=0; $ii<@$PCHREF;$ii++) {
			    if ($PCHREF->[$ii]) {
				push @PCHNUM, $ii;
				unless ($patcharg_p) {
				    $patcharg_p=' -p1' if &__check_patch_if_git_diff($spec->_get_sourcedir().'/'.basename($PCHREF->[$ii]));
				}
			    }
			}
			$patcharg_p//='';
			my $setupstring=join(' ','%setup -q',@setupargs)."\n";
			$setupstring='' if $autotype eq 'patch';
			my $patchesstring='';
			my $PATCHCONDCONTEXT=$spec->raw_patchcondcontext_ref;
			foreach my $patchnum (@PCHNUM) {
			    my ($patch_pre, $patch_post)=('','');
			    my $this_patch_condcontext=$PATCHCONDCONTEXT->[$patchnum];
			    if ($this_patch_condcontext) {
				my @condcontext=@$this_patch_condcontext;
				$patch_pre=join('',@condcontext);
				while (@condcontext>0) {
				    my $cond=pop @condcontext;
				    $patch_post .= '%endif'."\n" if $cond!~/^\%else/;
				}
			    }
			    $patchesstring .=$patch_pre.'%patch'.$patchnum.$patcharg_p."\n".$patch_post;
			}
			warn "INFO: auto$autotype: ----\n$setupstring$patchesstring\n----- END ----\n" if $verbose;
			$_=$autoprefix.$setupstring.$patchesstring."\n";
			return;
		    }
		    if (/(?<!%)\%\{(?:sources|patches)\}/) {
			my $SRCREF=$spec->raw_sourcelist_ref;
			my $PCHREF=$spec->raw_patchlist_ref;
			my @SRCNUM;
			for ($ii=0; $ii<@$SRCREF;$ii++) {
			    push @SRCNUM, $ii if $SRCREF->[$ii];
			}
			my @PCHNUM;
			for ($ii=0; $ii<@$PCHREF;$ii++) {
			    push @PCHNUM, $ii if $PCHREF->[$ii];
			}
			my $sourcesstring=join(' ',map {'%{SOURCE'.$_.'}'} @SRCNUM);
			my $patchesstring=join(' ',map {'%{PATCH'.$_.'}'} @PCHNUM);
			s/(?<!%)\%\{sources\}/$sourcesstring/g;
			s/(?<!%)\%\{patches\}/$patchesstring/g;
		    }
			       });
	} elsif ($t eq 'files') {
	    $section->map_body(
	sub {

	    # rpm404
	    # %caps(cap_ipc_lock=ep)
	    s/\%caps\([^\)]+\)//g; # not supported in alt rpm404

	    # not supported in rpm 404
	    # beware of prefix!
	    #%{!?_licensedir: %doc %{_pkgdocdir}/LICENSE}
	    #%{?_licensedir: %license LICENSE}
	    if (m/^(.*)\%(?:license|copying)(\s+)(\S.*)$/) {
		my ($prefix,$space,$suffix)=($1,$2,$3);
		my $expanded_string=$macros->macro_subst($suffix);
		#error: Can't mix special %doc with other forms: /usr/share/doc/slv2/COPYING
		# but we should allow %license %name/GPL
		if (substr($expanded_string,0,1) eq '/') { # was: $suffix=~m!^(?:/|\%)!
		    warn 'rpm2rpm404: INFO: ignored %license (Can\'t mix special %doc with other forms) in ',$suffix if $verbose;
		    $_=$prefix ? $prefix.$space.$suffix."\n" : $suffix."\n";
		} else {
		    $_=$prefix.'%doc --no-dereference'.$space.$suffix."\n";
		}
	    }
	    # {,} in %files, one-liners
	    if (m!^\s*\%\{?_libdir\}?/lib(.*)\.so\.\%\{?major\}?\{,\.\*\}\s*$!) {
		# %{_libdir}/lib%{name}.so.%{major}{,.*} -> not for rpm404
		$_='%{_libdir}/lib'.$1.'.so.%{major}*'."\n";
	    } elsif (m!\.\{py,pyc,pyo\}\s*$!) {
		# {,,} -> not for rpm404
		s!\.\{py,pyc,pyo\}(\s*)$!.py*$1!;
	    } elsif (m!^\s*\%doc\s*$!) {
		# %doc w/o args
		$_='%doc %_docdir/%name'."\n";
	    }
	
	    # rpm404 compat: {,} in %files, like
	    # like %{_ttffontdir}/*/CreteRound-{Italic,Regular}.otf
	    my $line=$_;
	    if (s/^(.*?)(?<!%)\{//) {
		my $prefix=$1;
		my $body='';
		while (s/^([^%\}]+|\%\{[^\}]+\})//) {
		    $body .= $1;
		}
		if (not s/\}//) {
		    $_=$line;
		    warn "rpm2rpm404: %files: can't unwrap line: $_";
		} else {
		    my $suffix=$_;
		    my @body=split(',',$macros->macro_subst($body));
		    $_=join('',map {$prefix.$_.$suffix} @body);
		}
	    }
			       });
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __check_patch_if_git_diff {
    my ($patchfile)=@_;
    return if ! -e $patchfile;
    open (RPMPATCH, '<', $patchfile) || die "can't open $patchfile: $!";
    local $_;
    my $is_git=0;
    while (<RPMPATCH>) {
	$is_git=1 if m/^--- a\//;
    }
    close (RPMPATCH);
    return $is_git;
}

sub alt_generic_cleanup_headers {
    my ($section)=@_;
    my @out;
    my $is_prev_line_multiline=0;
    foreach (@{$section->get_bodyref()}) {
	if (/^summary:/i) {
	    push @out, &__kill_non_ascii($_);
	} elsif (/^\s*#/i) {
	    #in comment: conakry-fonts-20070829-4.fc15/*.spec does not look like a spec file
	    push @out, &__kill_non_ascii($_);
	# TODO: mageia rpm5?
	} elsif (/^(?:VCS):/i) {
	    push @out, '#'.$_;
	} elsif (/^(?:Supplements|Enhances):/) {
	    print STDERR 'info: rpm2rpm404: dropped ',$_;
	    # exclude line
	} elsif (/\\\n$/ or $is_prev_line_multiline) {
	    # multiline macro for description: fonts-otf-beteckna
	    push @out, &__kill_non_ascii($_);
	} else {
	    push @out, $_;
	}
	$is_prev_line_multiline= /\\\n$/;
    }
    $section->set_body(\@out);
}

sub __kill_non_ascii {
    my ($line)=@_;
    return $line if not $opt_force_ascii;
    local $_=$line;
    # some distro use iso or utf8 in description. ALT Linux use ascii only. :(
    #tr//e/;
    tr/ޜ\x98\x99/-.....eooeaiaoucououaaidieaayvnieouabEOOEAIAOUCOUOUAAIDIEAAYVNIEOUAB''''/;
    # exterminatus of the rest of the non-ascii characters
    s/[\x80-\xff]/./g;
    if ($verbose and $line ne $_) {
	print STDERR "info: killed non-ascii for rpm404 in [$line]\n";
    }
    return $_;
}

# is macro %name used in spec before Name:,Version: tags
# it is not allowed in alt
sub alt_generic_rpm404_check_ahead {
    my ($spec,$section)=@_;
    my @out;
    my ($is_name_passed,$is_name_ahead);
    my ($is_version_passed,$is_version_ahead);
    my ($is_release_passed,$is_release_ahead);
    foreach (@{$section->get_bodyref()}) {
	m/^\s*name:/i and $is_name_passed=1; #and print STDERR "passed $_\n";
	m/^\s*version:/i and $is_version_passed=1; #and print STDERR "passed $_\n";
	m/^\s*release:/i and $is_release_passed=1; #and print STDERR "passed $_\n";
	$is_name_ahead=1 if not $is_name_passed and (m/\%\{name\}/ or m/\%name(?:[^\w\d_]|$)/);
	$is_version_ahead=1 if not $is_version_passed and (m/\%\{version\}/ or m/\%version(?:[^\w\d_]|$)/);
	$is_release_ahead=1 if not $is_release_passed and (m/\%\{release\}/ or m/\%release(?:[^\w\d_]|$)/);
	last if $is_name_passed and $is_version_passed and $is_release_passed;
    }
    if ($is_name_ahead or $is_version_ahead or $is_release_ahead) {
	my @predefs;
	push @predefs, '%%name' if $is_name_ahead;
	push @predefs, '%%version' if $is_version_ahead;
	push @predefs, '%%release' if $is_release_ahead;
	my $predefs_title=join(' and ',@predefs);
	my $main_section=$spec->main_section;
	$section->unshift_body(
	    "# $predefs_title is ahead of its definition. Predefining for rpm 4.0 compatibility.\n".
	    ($is_name_ahead ? '%define name '.$main_section->get_tag('Name')."\n" :'').
	    ($is_version_ahead ? '%define version '.$main_section->get_tag('Version')."\n" :'').
	    ($is_release_ahead ? '%define release '.$main_section->get_tag('Release')."\n" :'')
	    );
	$predefs_title=~s/\%\%/%/g;
	print STDERR "INFO: $predefs_title is ahead of its definition - fixed.\n";
    }
}

sub alt_generic_rpm404_single_f_flag_in_files {
    my ($spec)=@_;
    my $installsec=$spec->get_section('install');
    ### TODO
    my %base_filelists;
    # TODO - merge body here
    # my %base_sections;
    my $prev_section_found=0;
    foreach my $section ($spec->get_sections()) {
	my $t = $section->get_type();
	next if $t ne 'files';
	my @f_flags=$section->get_flag('-f');
	next unless @f_flags;
	# hack around old bug in get_glag
	next if @f_flags==1 and not defined($f_flags[0]);
	my $cname=$section->get_canonical_package();
	my $base_filelist=$base_filelists{$cname};
	if (defined $base_filelist) {
	    $prev_section_found=1;
	} else {
	    $base_filelist=shift @f_flags;
	    $base_filelists{$cname}=$base_filelist;
	    # $base_sections{$cname}=$section;
	    $prev_section_found=0;
	}
	if (@f_flags>0) {
	    my $cname_verbose=$cname ? $cname : '<main>';
	    unless ($installsec) {
		print STDERR "WARN: extra -f flags but no \%install section :-(\n";
	    } else {
		my $headlineref=$section->get_headline_ref();
		die "Oops! can't be: no neadline" unless $headlineref;
		foreach my $f_file (@f_flags) {
		    unless (defined $f_file) {
			die 'rpm2rpm404: merge multiple %files: internal error: undefined argumant to -f in %files for ',$cname_verbose. "\n", 'section: -f values are ',join(':',map {defined($_)? ($_ ? $_ : '<empty>') : '<undef>'} $section->get_flag('-f')),". body is:\n",$section->describe();
		    }
		    print STDERR "INFO: extra -f flags: merging file list $f_file into $base_filelist\n";
		    $installsec->push_body("# multiple -f flags in \%files for $cname_verbose: merging -f $f_file into -f $base_filelist
cat $f_file >> $base_filelist\n");
		    $$headlineref =~ s/-f\s+\Q$f_file\E//;
		}
		# destroy this section as the same section already exists.
		# see ant for example:
		#%files junit -f .mfiles-junit
		#%files junit -f .mfiles-junit4
		if ($prev_section_found) {
		    print STDERR "INFO: extra %files for $cname_verbose: merged and dropped\n";
		    $$headlineref='';
		}
	    }
	}
    }
}


sub opt_handler_rpm404_touch_ghosts {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    my (@ghosts,@ghostdirs);
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'files') {
	    $sec->map_body(sub {
		if (/(?<!%)\%ghost\s/) {
		    my $ghost=$_;
		    $ghost =~ s/\s*$//;
		    # %attr, %ghost, %...
		    $ghost =~ s/^.*(?<!\\)\s+//;
		    print STDERR "DEBUG: rpm404_touch_ghosts: found $ghost in $_" if $verbose>1;
		    if ($ghost=~m!/\s*$!) {
			push @ghostdirs, $ghost;
		    } else {
			push @ghosts, $ghost;
		    }
		}
	   });
	}
    }
    if (@ghosts>0 or @ghostdirs>0) {
	my $instsec=$spec->add_section('install');
	$instsec->push_body("\n",q!# touching all ghosts; hack for rpm 4.0.4!,"\n"),
	    $instsec->push_body(q!for rpm404_ghostdir in !.join(' ',@ghostdirs).q!
do
    mkdir -p %buildroot`dirname "$rpm404_ghostdir"`
done!."\n") if @ghostdirs;
	$instsec->push_body(q!for rpm404_ghost in !.join(' ',@ghosts).q!
do
    mkdir -p %buildroot`dirname "$rpm404_ghost"`
    touch %buildroot"$rpm404_ghost"
done!."\n") if @ghosts;
	$instsec->push_body("\n");
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;

package rpm2rpm404::DependencyFilter::RPM404DropReleaseInVersion;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    return unless $RPM::Source::Convert::Plugins::rpm2rpm404::rpm404;
    my $pkgversion=$dep->{-version};
    if ($pkgversion and $pkgversion=~m!-.+$! and $pkgversion!~m/-\%\{?release\}?$/
	# not = %(pkg-config --modversion libpari23 2>/dev/null || echo 0)
	and $pkgversion!~m/^\%\(/
	) {
	$pkgversion=~s!-.+$!!;
	$dep->{-version}=$pkgversion;
    }
    return $dep;
}

1;

__END__
